/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: HostMXBean.java 590 2011-11-02 14:14:18Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.api;

import java.util.List;
import java.util.Map;

import javax.management.InstanceNotFoundException;

import org.ow2.sirocco.vmm.api.monitoring.MonitorableMXBean;

/**
 * Management interface of a physical host acting as a container of virtual
 * machines by way of a hypervisor.
 * <p>
 * The <tt>ObjectName</tt> for identifying a HostMXBean follows the following
 * template: <blockquote>
 * org.ow2.sirocco.vmm.api:type=Host,name=&lt;pathname&gt;
 * ,hostname=&lt;hostname&gt; </blockquote>
 * <p>
 * The notifications that can be emitted by a HostMXBean are as follows
 * <ul>
 * <li>NotificationTypes.VM_NEW when a new VM is created
 * <li>NotificationTypes.VM_DEL when a VM is deleted
 * <li>NotificationTypes.PERF_REPORT: periodic performance reports
 * <li>NotificationTypes.HYPERVISOR_CONNECTION_STATE: when connection to host
 * hypervisor is established or lost
 * </ul>
 */
public interface HostMXBean extends ManagedResourceMXBean, MonitorableMXBean {
    /**
     * Different states of a host.
     */
    public enum HostPowerState {
        /**
         * the virtual machine is halted
         */
        HALTED,
        /**
         * The host is running
         */
        RUNNING,
        /**
         * the state of the host is not responding
         */
        NOT_RESPONDING,
        /**
         * the state of the host is unknown
         */
        UNKNOWN
    };

    /**
     * Returns true if the connection to the underlying driver-specific
     * hypervisor on the physical host is lost.
     * 
     * @return true if the connection to the host hypervisor is lost
     */
    @Deprecated
    public boolean isConnectionLost();

    /**
     * Returns the server pool this host belongs to
     * 
     * @return
     */
    ServerPoolMXBean getServerPool();

    /**
     * Returns the DNS name of the host
     * 
     * @return
     */
    String getHostName() throws VMMException;

    /**
     * Returns properties of the hypervisor active for this host
     * 
     * @return
     */
    Map<String, String> getHypervisorInfo() throws VMMException;

    /**
     * Return the number of CPU cores on the host
     * 
     * @return
     */
    int getNumCPU() throws VMMException;

    /**
     * Returns the information about the CPU on this host. The following
     * properties are returned: model, speed and vendor.
     * 
     * @return
     */
    Map<String, String> getCPUInfo() throws VMMException;

    /**
     * Returns the VMs resident on this host
     * 
     * @return
     */
    List<VirtualMachineMXBean> getResidentVMs() throws VMMException;

    /**
     * Returns the total memory capacity of this host in MBytes
     * 
     * @return
     */
    long getTotalMemoryMB() throws VMMException;

    /**
     * Returns the free memory capacity of this host in MBytes
     * 
     * @return
     */
    long getFreeMemoryMB() throws VMMException;

    /**
     * Returns the current load on each CPU core of the host
     * 
     * @return
     */
    float[] getLoadPerCPU() throws VMMException;

    /**
     * Returns the aggregate CPU load of the host
     * 
     * @return
     */
    float getCPULoad() throws VMMException;

    /**
     * Returns the power state of the host
     * 
     * @return current power state
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public HostPowerState getHostState() throws VMMException;

    /**
     * Return the CPU load of every VM hosted on this host
     * 
     * @return a map of CPU loads, as percentages, keyed by VM labels
     */
    Map<String, Float> getVMCPULoads() throws VMMException;

    /**
     * Creates a new virtual machine on this host. Upon successful creation a
     * notification of type NotificationTypes.VM_NEW will be emitted by the
     * HostMXBean on which the VM has been created.
     * 
     * @param vmSpec initial configuration of the virtual machine
     * @param sync if true the call will return once the VM has been created
     * @return the management interface of the created VM if <tt>sync</tt> is
     *         true, null otherwise
     * @throws InsufficientResourcesException raised if not enough resources are
     *         available to create the VM
     * @throws InvalidVMConfigException raised if the VM configuration is
     *         invalid
     */
    @Deprecated
    VirtualMachineMXBean createVM(VMConfigSpec vmSpec, boolean sync) throws InsufficientResourcesException,
        InvalidVMConfigException, VMMException;

    @Deprecated
    VirtualMachineMXBean createVM(VMConfigSpec vmSpec, ResourcePartitionMXBean partition, boolean sync)
        throws InsufficientResourcesException, InvalidVMConfigException, VMMException;

    /**
     * Creates a new virtual machine. The host on which the VM will be created
     * will depend on the placement policy active for Server Pool.
     * 
     * @param vmConfigSpec the configuration of the virtual machine hardware
     *        (CPU, memory, devices)
     * @param customizationSpec customization specification of the guest OS or
     *        null if no customization required
     * @param startVM if true the VM is started once created
     * @param sync if true the call will return once the VM has been created
     * @return the management interface of the created VM if <tt>sync</tt> is
     *         true, null otherwise
     * @throws InsufficientResourcesException raised if not enough resources are
     *         available to create the VM
     * @throws InvalidVMConfigException raised if the VM configuration is
     *         invalid
     * @throws VMMException raised if any runtime error occurs during the VM
     *         creation
     */
    VirtualMachineMXBean createVirtualMachine(VirtualMachineConfigSpec vmConfigSpec, CustomizationSpec customizationSpec,
        boolean startVM, boolean sync) throws InsufficientResourcesException, InvalidVMConfigException, VMMException;

    /**
     * shutdown the host
     */
    public void stop() throws VMMException;

    /**
     * start the host
     */
    public void start() throws VMMException;

    /**
     * Returns the VM image store associated with this host
     * 
     * @return
     */
    @Deprecated
    VirtualMachineImageStoreMXBean getVMImageStore() throws VMMException;

    /**
     * Returns the current resource usage (CPU, memory, network, disk I/O) for
     * all VM residing on this host
     * 
     * @return a map of ResourceUsage keyed by the VM label
     */
    Map<String, ResourceUsage> getVMResourceUsage() throws VMMException;

}
