/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ResourcePartitionMXBean.java 72 2011-06-29 07:53:22Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.api;

import java.util.List;
import java.util.Map;

/**
 * Management interface of a resource partition. A resource partition represents
 * a subset of the resources (CPU, memory, storage) spanning the hosts of a
 * server pool. A resource partition is configured with a specific capacity
 * (number of CPU cores, memory size and storage space)) that corresponds to a
 * resource reservation. A resource partition can be subdivided by creating
 * child resource partitions. A server pool is a resource partition whose
 * capacity is the aggregate capacity of the hosts and shared storage of the
 * pool.
 * <p>
 * The <tt>ObjectName</tt> for identifying a ResourcePartitionMXBean follows the
 * following template: <blockquote>
 * org.ow2.sirocco.vmm.api:type=ResourcePartition,name=&lt;pathname&gt;
 * ,name=&lt;name&gt; </blockquote>
 * <p>
 */
public interface ResourcePartitionMXBean extends ManagedResourceMXBean {
    /**
     * Returns the symbolic name of the resource partition
     * 
     * @return
     */
    String getName();

    /**
     * Returns the CPU capacity of the partition expressed as a number of cores
     * 
     * @return
     */
    long getCpuCoreCapacity();

    /**
     * Returns the memory capacity in megabytes of the partition
     * 
     * @return
     */
    long getMemoryCapacityMB();

    /**
     * Returns the storage capacity in megabytes of the partition
     * 
     * @return
     */
    long getStorageCapacityMB();

    /**
     * Returns the free CPU capacity of the partition expressed as a number of
     * cores
     * 
     * @return
     */
    long getFreeCpuCoreCapacity();

    /**
     * Returns the free memory capacity of the partition in megabytes
     * 
     * @return
     */
    long getFreeMemoryCapacityMB();

    /**
     * Returns the free storage capacity of the partition in megabytes
     * 
     * @return
     */
    long getFreeStorageMBCapacity();

    /**
     * Sets the CPU over-provisioning ratio, i.e. the ratio between the number
     * of virtual CPU allocated to VMs and the number of physical CPUs of the
     * partitions
     * 
     * @param ratio number of virtual CPUs allocated to one physical CPU
     * @param propagate if true propagate the setting to child partitions
     * @throws InsufficientResourcesException raised if the the vCPU/pCPU ratio
     *         cannot accommodate the partition VMs and sub-partitions
     */
    void setCpuOverprovisioningRatio(int ratio, boolean propagate) throws InsufficientResourcesException;

    /**
     * Returns the the CPU over-provisioning ration, i.e. the ratio between the
     * number of virtual CPU allocated to VMs and the number of physical CPUs of
     * the partitions
     * 
     * @return
     */
    int getCpuOverprovisioningRatio();

    /**
     * Returns properties of the resource partition
     * 
     * @return
     */
    Map<String, String> getProperties();

    /**
     * Creates a new virtual machine within the resource partition. The host on
     * which the VM will be created will depend on the placement policy active
     * for the root server pool of the partition. Upon successful creation a
     * notification of type NotificationTypes.VM_NEW will be emitted by the
     * HostMXBean on which the VM has been created.
     * 
     * @param vmSpec initial configuration of the virtual machine
     * @param constraints constraints on the placement of the VM
     * @param sync if true the call will return once the VM has been created
     * @return the management interface of the created VM if <tt>sync</tt> is
     *         true, null otherwise
     * @throws InsufficientResourcesException raised if not enough resources are
     *         available to create the VM within the partition
     * @throws InvalidVMConfigException raised if the VM configuration is
     *         invalid
     */
    @Deprecated
    VirtualMachineMXBean provisionVM(final VMConfigSpec vmSpec, final Map<String, String> constraints, boolean sync)
        throws InsufficientResourcesException, InvalidVMConfigException, VMMException;

    /**
     * Creates a new virtual machine. The host on which the VM will be created
     * will depend on the placement policy active for Server Pool.
     * 
     * @param vmConfigSpec the configuration of the virtual machine hardware
     *        (CPU, memory, devices)
     * @param customizationSpec customization specification of the guest OS or
     *        null if no customization required
     * @param startVM if true the VM is started once created
     * @param sync if true the call will return once the VM has been created
     * @return the management interface of the created VM if <tt>sync</tt> is
     *         true, null otherwise
     * @throws InsufficientResourcesException raised if not enough resources are
     *         available to create the VM
     * @throws InvalidVMConfigException raised if the VM configuration is
     *         invalid
     * @throws VMMException raised if any runtime error occurs during the VM
     *         creation
     */
    VirtualMachineMXBean createVirtualMachine(VirtualMachineConfigSpec vmConfigSpec, CustomizationSpec customizationSpec,
        boolean startVM, boolean sync) throws InsufficientResourcesException, InvalidVMConfigException, VMMException;

    /**
     * Returns the virtual machines belonging to this partition
     * 
     * @return
     * @throws VMMException raised if a runtime error occurs
     */
    List<VirtualMachineMXBean> getVirtualMachines() throws VMMException;

    /**
     * Moves a virtual machine into this partition. This operation is allowed
     * only if the original resource partition of the VM has the same root
     * server pool as the target partition
     * 
     * @param vm the virtual machine to move
     * @throws InsufficientResourcesException raised if the target partition has
     *         not enough resources to host the VM
     * @throws IllegalOperationException raised if the VM does not belong to the
     *         same server pool
     * @throws VMMException raised if a runtime error occurs
     */
    void moveIntoResourcePartition(VirtualMachineMXBean vm) throws InsufficientResourcesException, IllegalOperationException,
        VMMException;

    /**
     * Changes the resource capacity of the partition.
     * 
     * @param cpuCoreCapacity new CPU capacity expressed as a number of CPU
     *        cores
     * @param memoryCapacityMB new memory capacity in megabytes
     * @param storageCapacityMB new storage capacity in megabytes
     * @throws InsufficientResourcesException raised if the parent partition has
     *         not enough free resources to accommodate the growing partition or
     *         if the shrinking capacity is not big enough for the virtual
     *         machines and child partitions
     */
    void changeCapacity(long cpuCoreCapacity, long memoryCapacityMB, long storageCapacityMB)
        throws InsufficientResourcesException, IllegalOperationException;

    /**
     * Creates a child resource partition
     * 
     * @param name symbolic name of the partition
     * @param cpuCores CPU capacity of the child partition
     * @param memoryCapacityMB memory capacity on megabytes of the child
     *        partition
     * @param storageMB storage capacity in megabytes of the child partition
     * @param properties driver-specific properties
     * @return the management interface of the created resource partition
     * @throws InsufficientResourcesException raised if the free capacity of the
     *         parent partition cannot accommodate the child partition
     * @throws VMMException
     */
    ResourcePartitionMXBean createSubResourcePartition(String name, long cpuCores, long memoryCapacityMB, long storageMB,
        Map<String, String> properties) throws InsufficientResourcesException, VMMException;

    /**
     * Returns the sub-partitions of this partition
     * 
     * @return
     */
    List<ResourcePartitionMXBean> getSubResourcePartitions();

    /**
     * Returns the parent resource partition or null. Note that only server
     * pools have no parent partition
     * 
     * @return
     */
    ResourcePartitionMXBean getParent();

    /**
     * Returns the root server pool this resource partition belongs to
     * 
     * @return
     */
    ServerPoolMXBean getRootServerPool();

    /**
     * Removes the specified resource partition. This entails the recursive
     * destruction of all child partitions of the specified partition. All
     * virtual machines belonging to the specified resource partition (or to one
     * of its child partitions) are moved up to the current partition.
     * 
     * @param partition the sub-partition to remove
     * @throws VMMException raised if a runtime error occurs
     */
    void removeSubResourcePartition(ResourcePartitionMXBean partition) throws VMMException;
}
