/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ServerPoolMXBean.java 697 2011-12-08 17:21:32Z dangtran $
 *
 */

/**
 /**
 * JASMINe VMMapi: JASMINe Virtual Machine Management API
 * Copyright (C) 2009-2010 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerPoolMXBean.java 697 2011-12-08 17:21:32Z dangtran $
 * --------------------------------------------------------------------------
 */
package org.ow2.sirocco.vmm.api;

import java.util.List;
import java.util.Map;

import org.ow2.sirocco.vmm.api.monitoring.MonitorableMXBean;

/**
 * Management interface of a Server Pool. A Server Pool is a management unit
 * acting as a container of physical hosts and virtual machines. A Server Pool
 * is homogeneous with regard to the virtualization technology used by its
 * member hosts. Live or cold migration of virtual machines between hosts is
 * allowed only if the source and target host belong to the same server pool.
 * <p>
 * The <tt>ObjectName</tt> for identifying a ServerPoolMXBean follows the
 * following template: <blockquote>
 * org.ow2.sirocco.vmm.api:type=ServerPool,name=&lt;pathname&gt; </blockquote>
 * <p>
 */
public interface ServerPoolMXBean extends ResourcePartitionMXBean, MonitorableMXBean {
    /**
     * Return the symbolic name of the server pool
     * 
     * @return
     */
    String getName();

    /**
     * Sets the provisioning policy of this domain
     * 
     * @param policyId
     */
    void setProvisioningPolicy(String policyId);

    /**
     * Returns the physical hosts belong to the server pool
     * 
     * @return
     */
    List<HostMXBean> getManagedHosts();

    /**
     * Returns the shared storage pools associated with this server pool and
     * accessible by the hosts belonging to this server pool
     * 
     * @return
     */
    List<StoragePoolMXBean> getStoragePools();

    /**
     * Adds a shared storage pool to the server pool
     * 
     * @param name symbolic name of the storage pool
     * @param attributes driver-specific attributes of the storage pool
     * @throws VMMException raised of the operation fails
     */
    void addStoragePool(String name, Map<String, String> attributes) throws VMMException;

    /**
     * Removes a storage pool from the server pool
     * 
     * @param storagePool storage pool to remove
     * @throws VMMException raised if the operation failed
     */
    void removeStoragePool(StoragePoolMXBean storagePool) throws VMMException;

    /**
     * Returns the image stores belonging to this server pool
     * 
     * @return
     */
    @Deprecated
    VirtualMachineImageStoreMXBean getVMImageStore();

    /**
     * Add a new Host to the server pool
     * 
     * @param attributes attributes of the new host
     * @return the added hostMXBean
     */
    HostMXBean addHost(final Map<String, String> attributes) throws VMMException;

    /**
     * Delete a host from the server pool
     * 
     * @param host host to delete
     */
    void deleteHost(HostMXBean host) throws VMMException;

    /**
     * Returns the hypervisor common to every host of this pool
     * 
     * @return hypervisor type
     */
    String getHypervisor();

    /**
     * move a host to a different server pool
     * 
     * @param host host to delete
     * @param pool destination pool
     */
    void moveHost(final HostMXBean host, final ServerPoolMXBean pool) throws VMMException;

    /**
     * Returns the VM placement rules for this pool
     * @return list of VM placement rules
     */
    List<VMPlacementRule> getVMPlacementRules();

    /**
     * Creates or update a VM placement rule
     * @param name name of the rule, if a rule with this name already exists, the operation will update it otherwise a new rule is created
     * @param type type of the rule
     * @param predicate predicate of the rule
     * @throws VMMException raised if the rule cannot be created or updated
     */
    void createVMPlacementRule(String name, VMPlacementRule.Type type, String predicate) throws VMMException;

    /**
     * Removes a VM placement rule
     * @param name name of the rule to remove
     * @throws VMMException raised if the rule cannot be removed
     */
    void removeVMPlacementRule(String name) throws VMMException;
}
