/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: StoragePoolMXBean.java 590 2011-11-02 14:14:18Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.api;

import java.util.List;

/**
 * Management interface of a StoragePool. A StoragePool is a container of
 * volumes which can be attached to virtual machines accessible from this
 * storage pool.
 */
public interface StoragePoolMXBean extends ManagedResourceMXBean {

    /**
     * Returns the name of the storage pool
     * 
     * @return
     */
    String getName();

    /**
     * Returns the supported disk image formats
     * 
     * @return
     */
    String[] getSupportedDiskImageFormats();

    /**
     * Returns the capacity of the storage pool in megabytes
     * 
     * @return the capacity of the storage pool in megabytes
     */
    long getCapacityMB() throws VMMException;

    /**
     * Returns the free space remaining on the storage pool in megabytes
     * 
     * @return the free space remaining on the storage pool in megabytes
     */
    long getFreeSpaceMB() throws VMMException;

    /**
     * Returns the storage type of the StoragePool
     * 
     * @return the storage type of the StoragePool
     */
    String getType();

    /**
     * Returns the volumes contained by this pool
     * 
     * @return the volumes contained by this pool
     * @throws VMMException if an error occurs while retrieving the volumes
     */
    List<Volume> getVolumes() throws VMMException;

    /**
     * Returns a volume matching a given key
     * 
     * @param key unique key of the volume to return
     * @return volume matching the key
     */
    Volume getVolumeByKey(String key);

    /**
     * Destroys a volumes contained in this pool
     * 
     * @param key the key of the volume
     * @throws VMMException if an error occurs while destroying the volume
     */
    void destroyVolume(String key) throws VMMException;

    /**
     * Creates a new empty volume on this pool
     * 
     * @param name name of the volume
     * @param capacityMB capacity of the volume in megabytes
     * @return the new volume instance
     * @throws VMMException if an error occurs while creating the volume
     */
    Volume createVolume(String name, long capacityMB, String format) throws VMMException;

    /**
     * Creates a new volume based on an existing parent volume
     * 
     * @param name name of the volume
     * @param capacityMB capacity of the volume in megabytes. Note that this
     *        size cannot be less than the size of parent volume
     * @param parentVolume the volume to be used to initialize the new volume
     * @param copyOnWrite if true create the volume as a copy-on-write clone of
     *        the parent volume, otherwise make a copy
     * @return the new volume instance
     * @throws VMMException if an error occurs while creating the volume
     */
    Volume createVolumeFrom(String name, long capacityMB, String format, Volume parentVolume, boolean copyOnWrite)
        throws VMMException;

    /**
     * Uploads a new volume
     * 
     * @param url url of the volume file
     * @param format format of the volume
     * @param name name of the volume
     * @param description description of the disk image
     * @param checkSum SHA1 checksum of the file
     * @return new Volume instance
     * @throws InsufficientResourcesException if the image store is full
     * @throws VMMException if an error occurs
     */
    Volume uploadVolume(String url, String format, String name, String description, String checkSum)
        throws InsufficientResourcesException, VMMException;

    /**
     * Refreshes the storage pool
     */
    public void refresh();
}
