/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VirtualMachineConfigSpec.java 698 2011-12-08 17:21:59Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.api;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Helper class used to specify the configuration settings when creating a
 * virtual machine
 */
public class VirtualMachineConfigSpec implements Serializable {
    private static final long serialVersionUID = -3652438739216789028L;

    private String name;

    private int numVCPUs;

    private long memoryMB;

    private List<VirtualDiskSpec> diskSpecs = new ArrayList<VirtualDiskSpec>();

    private List<VirtualCdromSpec> cdromSpecs = new ArrayList<VirtualCdromSpec>();

    private List<VNICSpec> vnicSpecs = new ArrayList<VNICSpec>();

    private ResourcePartitionMXBean partition;

    private Map<String, String> properties = new HashMap<String, String>();

    private Map<String, String> userMetadata = new HashMap<String, String>();

    public VirtualMachineConfigSpec() {

    }

    public VirtualMachineConfigSpec(final VirtualMachineConfigSpec other) {
        this.name = other.name;
        this.numVCPUs = other.numVCPUs;
        this.memoryMB = other.memoryMB;
        this.diskSpecs = other.diskSpecs;
        this.cdromSpecs = other.cdromSpecs;
        this.vnicSpecs = other.vnicSpecs;
        this.partition = other.partition;
        this.properties = other.properties;
        this.userMetadata = other.userMetadata;
    }

    /**
     * Returns the symbolic name of the virtual machine
     * 
     * @return the symbolic name of the virtual machine
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the symbolic name of the virtual machine
     * 
     * @param name the symbolic name of the virtual machine
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * Returns the number of virtual CPUs of the virtual machine
     * 
     * @return the number of virtual CPUs of the virtual machine
     */
    public int getNumVCPUs() {
        return this.numVCPUs;
    }

    /**
     * Sets the number of virtual CPUs of the virtual machine
     * 
     * @param numVCPUs the number of virtual CPUs of the virtual machine
     */
    public void setNumVCPUs(final int numVCPUs) {
        this.numVCPUs = numVCPUs;
    }

    /**
     * Returns the size of the VM's memory in megabytes
     * 
     * @return the size of the VM's memory in megabytes
     */
    public long getMemoryMB() {
        return this.memoryMB;
    }

    /**
     * Sets the size of the VM's memory in megabytes
     * 
     * @param memoryMB the size of the VM's memory in megabytes
     */
    public void setMemoryMB(final long memoryMB) {
        this.memoryMB = memoryMB;
    }

    public List<VirtualDiskSpec> getDiskSpecs() {
        return this.diskSpecs;
    }

    public void setDiskSpecs(final List<VirtualDiskSpec> diskSpecs) {
        this.diskSpecs = diskSpecs;
    }

    public List<VirtualCdromSpec> getCdromSpecs() {
        return this.cdromSpecs;
    }

    public void setCdromSpecs(final List<VirtualCdromSpec> cdromSpecs) {
        this.cdromSpecs = cdromSpecs;
    }

    public List<VNICSpec> getVnicSpecs() {
        return this.vnicSpecs;
    }

    public void setVnicSpecs(final List<VNICSpec> vnicSpecs) {
        this.vnicSpecs = vnicSpecs;
    }

    /**
     * Returns the resource partition in which the VM should be created
     * 
     * @return
     */
    public ResourcePartitionMXBean getPartition() {
        return this.partition;
    }

    /**
     * Sets the resource partition in which the VM should be created
     * 
     * @param partition
     */
    public void setPartition(final ResourcePartitionMXBean partition) {
        this.partition = partition;
    }

    /**
     * Returns additional configuration settings
     * 
     * @return additional configuration settings
     */
    public Map<String, String> getProperties() {
        return this.properties;
    }

    /**
     * Sets additional configuration settings
     * 
     * @param properties additional configuration settings
     */
    public void setProperties(final Map<String, String> properties) {
        this.properties = properties;
    }

    public Map<String, String> getUserMetadata() {
        return this.userMetadata;
    }

    public void setUserMetadata(final Map<String, String> userMetadata) {
        this.userMetadata = userMetadata;
    }

    public int computeDiskSpaceRequirementMB() {
        int spaceMB = 0;
        for (VirtualDiskSpec deviceSpec : this.diskSpecs) {
            spaceMB += deviceSpec.getCapacityMB();
        }
        return spaceMB;
    }

    @Override
    public String toString() {
        return "VirtualMachineConfigSpec [name=" + this.name + ", numVCPUs=" + this.numVCPUs + ", memoryMB=" + this.memoryMB
            + ", diskSpecs=" + this.diskSpecs + ", cdromSpecs=" + this.cdromSpecs + ", vnicSpecs=" + this.vnicSpecs
            + ", partition=" + this.partition + ", properties=" + this.properties + "]";
    }

}
