/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VirtualMachineMXBean.java 699 2011-12-08 17:22:27Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.api;

import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.management.InstanceNotFoundException;

import org.ow2.sirocco.vmm.api.monitoring.MonitorableMXBean;

/**
 * Management interface of a virtual machine. The <tt>ObjectName</tt> for
 * identifying a VirtualMachineMXBean follows the following template:
 * <p>
 * <code>
 *    org.ow2.sirocco.vmm.api:type=VirtualMachine,name=&lt;pathname&gt;,uuid=&lt;uuid&gt;
 * </code>
 * <p>
 * The notifications that can be emitted by a HostMXBean are as follows
 * <ul>
 * <li>NotificationTypes.VM_MIGRATION when the VM has migrated to a new host
 * <li>NotificationTypes.VM_STATE_CHANGE when the power state of the VM has
 * changed
 * </ul>
 */
public interface VirtualMachineMXBean extends ManagedResourceMXBean, MonitorableMXBean {
    /**
     * Different states of a virtual machine.
     */
    public enum PowerState {
        /**
         * the virtual machine is halted
         */
        HALTED,
        /**
         * the virtual machine is paused
         */
        PAUSED,
        /**
         * The virtual machine is running
         */
        RUNNING,
        /**
         * the virtual machine is suspended
         */
        SUSPENDED,
        /**
         * the state of the virtual machine is unknown
         */
        UNKNOWN
    };

    /**
     * Returns the Uuid of the virtual machine
     * 
     * @return
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public String getUuid() throws VMMException;

    /**
     * Returns the host this VM resides on
     * 
     * @return
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public HostMXBean getHostMBean() throws VMMException;

    /**
     * Returns the name of the virtual machine
     * 
     * @return
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public String getNameLabel() throws VMMException;

    /**
     * Returns the MAC addresses of all network interfaces associated with this
     * VM
     * 
     * @return MAC addresses in xx:xx:xx:xx:xx:xx format
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public String getMacAddress() throws VMMException;

    /**
     * Returns the IP addresses of the guest operating system running within the
     * VM.
     * 
     * @return IP addresses or null if the manager is unable to determine the IP
     *         addresses
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public String getGuestIpAddress() throws VMMException;

    /**
     * Returns information about the guest operating system
     * 
     * @return
     * @throws VMMException
     */
    public GuestInfo getGuestInfo() throws VMMException;

    /**
     * Returns the load per virtual CPU of the VM
     * 
     * @return
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public float[] getLoadPerVCPU() throws VMMException;

    /**
     * Returns the CPU load of the VM
     * 
     * @return
     * @throws InstanceNotFoundException
     */
    public float getCPULoad() throws VMMException;

    /**
     * Returns the start time of the virtual machine
     * 
     * @return
     * @throws InstanceNotFoundException
     */
    public Date getStartTime() throws VMMException;

    /**
     * Returns the tags attached to the virtual machine
     * 
     * @return
     */
    public Map<String, String> getUserMetadata();

    /**
     * Add or update tags attached to the virtual machine
     * 
     * @param metadata tags to add or update
     */
    public void createUserMetadata(Map<String, String> metadata) throws VMMException;

    /**
     * Returns the resource partition this virtual machine belongs to
     * 
     * @return
     */
    public ResourcePartitionMXBean getResourcePartition();

    /**
     * Returns the memory size in MBytes of the virtual machine
     * 
     * @return
     * @throws InstanceNotFoundException
     */
    public long getMemorySizeMB() throws VMMException;

    /**
     * Attempts to change the memory sie of the virtual machine
     * 
     * @param size target size
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public void setMemorySizeMB(long size) throws VMMException;

    /**
     * Returns the amount of memory (in MBytes) used by the virtual machine
     * 
     * @return
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public long getMemoryUsedMB() throws VMMException;

    /**
     * Return the current resource usage (CPU, memory, network, disk I/O) for
     * this VM
     * 
     * @return a ResourceUsage object representing the resource usage of this VM
     */
    public ResourceUsage getResourceUsage() throws VMMException;

    /**
     * Returns the number of virtual CPUs assigned to the virtual machine
     * 
     * @return
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public int getNumVCPUs() throws VMMException;

    /**
     * Changes the number of virtual CPUs assigned to the virtual machine
     * 
     * @param numVCPUs
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public void setNumVCPUs(int numVCPUs) throws VMMException;

    /**
     * Returns the scheduling cap assigned to the VM
     * 
     * @return current scheduling cap
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public int getSchedulingCap() throws VMMException;

    /**
     * Sets the scheduling cap assigned to the VM
     * 
     * @param schedulingCap new cap
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public void setSchedulingCap(int schedulingCap) throws VMMException;

    /**
     * Returns the scheduling weight assigned to the VM
     * 
     * @return
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public int getSchedulingWeight() throws VMMException;

    /**
     * Sets the scheduling cap assigned to the VM
     * 
     * @param schedulingWeight new weight
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public void setSchedulingWeight(int schedulingWeight) throws VMMException;

    /**
     * Returns the CPU affinity for each VCPU of the virtual machine The
     * affinity is represented by a matrix indexed by the VCPU number (first
     * array dimension) and by the physical CPU number (second array dimension).
     * A true value for element [i][j] signifies that VCPU i can be mapped to
     * the physical CPU j of the underlying host
     * 
     * @return the CPU affinity for all VCPUs of the virtual machine
     */
    public boolean[][] getCPUAffinity() throws VMMException;

    /**
     * Sets the CPU affinity for each VCPU of the virtual machine The affinity
     * is represented by a matrix indexed by the VCPU number (first array
     * dimension) and by the physical CPU number (second array dimension). A
     * true value for element [i][j] signifies that VCPU i can be mapped to the
     * physical CPU j of the underlying host. Note that a particular
     * virtualization system might not be able to enforce this CPU affinity or
     * only partially.
     * 
     * @param affinity the CPU affinity
     */
    public void setCPUAffinity(boolean[][] affinity) throws VMMException;

    /**
     * Returns the power state of the VM
     * 
     * @return current power state
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public PowerState getState() throws VMMException;

    /**
     * Suspend the VM
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws BadVMPowerStateException thrown if the VM is not in the
     *         appropriate state
     */
    public void suspend() throws VMMException, BadVMPowerStateException;

    /**
     * Resume the VM
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws BadVMPowerStateException thrown if the VM is not in the
     *         appropriate state
     */
    public void resume() throws VMMException, BadVMPowerStateException;

    /**
     * Shutdown the VM
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws BadVMPowerStateException thrown if the VM is not in the
     *         appropriate state
     */
    public void shutdown() throws VMMException, BadVMPowerStateException;

    /**
     * Start the VM
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws BadVMPowerStateException thrown if the VM is not in the
     *         appropriate state
     */
    public void start() throws VMMException, BadVMPowerStateException;

    /**
     * Pause the VM
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws BadVMPowerStateException thrown if the VM is not in the
     *         appropriate state
     */
    public void pause() throws VMMException, BadVMPowerStateException;

    /**
     * Unpause the VM
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws BadVMPowerStateException thrown if the VM is not in the
     *         appropriate state
     */
    public void unpause() throws VMMException, BadVMPowerStateException;

    /**
     * Reboot the VM
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws BadVMPowerStateException thrown if the VM is not in the
     *         appropriate state
     */
    public void reboot() throws VMMException, BadVMPowerStateException;

    /**
     * Destroy the VM and all its virtual disks
     * 
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    @Deprecated
    public void destroy() throws VMMException;

    /**
     * Destroy the VM
     * 
     * @param deleteDisks if true delete all virtual disks of the VM, otherwise
     *        leave them untouched
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     */
    public void destroy(boolean deleteDisks) throws VMMException;

    /**
     * Returns true if the VM can be migrated to the given host using live
     * migration
     * 
     * @param targetHost host on which the VM migration is tested
     */
    public boolean canLiveMigrateToHost(HostMXBean targetHost) throws VMMException;

    /**
     * Migrate the VM to a target host. Note that the target host must belong to
     * the same server pool as the VM's current host.
     * 
     * @param targetHost target host of the migration
     * @param live if true a live migration will be attempted
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws IllegalOperationException thrown if the migration cannot be
     *         performed
     */
    public void migrate(HostMXBean targetHost, boolean live) throws IllegalOperationException, VMMException;

    /**
     * Returns the console of the virtual machine (if supported by the driver
     * and exposed by the virtual machine)
     * 
     * @return console of the virtual machine or null if none available
     */
    public String getConsole() throws VMMException;

    /**
     * Clones the VM
     * 
     * @param vmName name to be given to the new VM
     * @param custSpec optional customization parameters
     * @param sync if true, the operation is synchronous
     * @return the management interface of the cloned VM if <tt>sync</tt> is
     *         true, null otherwise
     * @throws InstanceNotFoundException thrown if the specified MXBean does not
     *         exist
     * @throws InsufficientResourcesException raised if not enough resources are
     *         available to clone the VM
     */
    public VirtualMachineMXBean cloneVM(String vmName, VMCustomizationSpec custSpec, boolean sync)
        throws InsufficientResourcesException, VMMException;

    /**
     * Creates a new VM image out of this virtual machine This operation is
     * allowed only if the VM is not running
     * 
     * @param name name of the image
     * @param description description of the image
     * @param metadata metadata to associate with the image
     * @return the created virtual image
     * @throws InsufficientResourcesException raised if there is not enough
     *         storage capacity for the new image
     * @throws IllegalOperationException raised if the virtual machine is not
     *         connected to this VirtualMachineImageStore
     * @throws BadVMPowerStateException raised if the VM is running
     * @throws VMMException raised if a runtime error occurs
     */
    VirtualMachineImageMXBean makeTemplate(String name, String description, Map<String, String> metadata)
        throws InsufficientResourcesException, IllegalOperationException, BadVMPowerStateException, VMMException;

    Volume makeImage(String name, StoragePoolMXBean storagePool) throws VMMException;

    /**
     * Returns the virtual disks attached to this virtual machine
     * 
     * @return the virtual disks attached to this virtual machine
     */
    List<VirtualDisk> getVirtualDisks();

    /**
     * Returns the virtual CD-ROMs attached to this virtual machine
     * 
     * @return the virtual CD-ROMs attached to this virtual machine
     */
    List<VirtualCdrom> getVirtualCdroms();

    /**
     * Attaches a volume to this virtual machine
     * 
     * @param volume volume to attach
     * @param device name of the device
     * @throws VMMException if the attachment fails
     */
    void attachVolume(VirtualDiskSpec diskSpec) throws VMMException;

    /**
     * Detaches a volume from the virtual machine
     * 
     * @param key unique key of the volume
     * @throws VMMException if the operation fails
     */
    void detachVolume(String key) throws VMMException;

    /**
     * Changes the media file of an existing CDROM device
     * 
     * @param device device name of the CDROM
     * @param iso ISO image
     * @throws VMMException if a runtime error occurs
     */
    void changeCdromMedia(String device, Volume iso) throws VMMException;

    /**
     * Returns the VNICs attached to this virtual machine
     * 
     * @return the VNICs attached to this virtual machine
     */
    List<VNIC> getVNICs();
}
